@@ -121,18 +121,21 @@ impl<T: Float, F: Real> AudioNode for Biquad<T, F> {
 }
 
 /// Butterworth lowpass filter.
+/// Number of inputs is `N`, either `U1` or `U2`.
 /// - Input 0: input signal
-/// - Input 1: cutoff frequency (Hz)
+/// - Input 1 (optional): cutoff frequency (Hz)
 /// - Output 0: filtered signal
-pub struct ButterLowpass<T: Float, F: Real> {
+pub struct ButterLowpass<T: Float, F: Real, N: Size<T>> {
+    _marker: std::marker::PhantomData<N>,
     biquad: Biquad<T, F>,
     sample_rate: F,
     cutoff: F,
 }
 
-impl<T: Float, F: Real> ButterLowpass<T, F> {
+impl<T: Float, F: Real, N: Size<T>> ButterLowpass<T, F, N> {
     pub fn new(sample_rate: f64, cutoff: F) -> Self {
         let mut node = ButterLowpass {
+            _marker: std::marker::PhantomData::default(),
             biquad: Biquad::new(),
             sample_rate: F::from_f64(sample_rate),
             cutoff: F::zero(),
@@ -147,10 +150,10 @@ impl<T: Float, F: Real> ButterLowpass<T, F> {
     }
 }
 
-impl<T: Float, F: Real> AudioNode for ButterLowpass<T, F> {
+impl<T: Float, F: Real, N: Size<T>> AudioNode for ButterLowpass<T, F, N> {
     const ID: u64 = 16;
     type Sample = T;
-    type Inputs = typenum::U2;
+    type Inputs = N;
     type Outputs = typenum::U1;
 
     fn reset(&mut self, sample_rate: Option<f64>) {
@@ -166,9 +169,11 @@ impl<T: Float, F: Real> AudioNode for ButterLowpass<T, F> {
         &mut self,
         input: &Frame<Self::Sample, Self::Inputs>,
     ) -> Frame<Self::Sample, Self::Outputs> {
-        let cutoff: F = convert(input[1]);
-        if cutoff != self.cutoff {
-            self.set_cutoff(cutoff);
+        if N::USIZE > 1 {
+            let cutoff: F = convert(input[1]);
+            if cutoff != self.cutoff {
+                self.set_cutoff(cutoff);
+            }
         }
         self.biquad.tick(&[input[0]].into())
     }
@@ -187,20 +192,23 @@ impl<T: Float, F: Real> AudioNode for ButterLowpass<T, F> {
 
 /// Constant-gain bandpass filter (resonator).
 /// Filter gain is (nearly) independent of bandwidth.
+/// Number of inputs is `N`, either `U1` or `U3` (sic).
 /// - Input 0: input signal
-/// - Input 1: filter center frequency (peak) (Hz)
-/// - Input 2: filter bandwidth (distance) between -3 dB points (Hz)
+/// - Input 1 (optional): filter center frequency (peak) (Hz)
+/// - Input 2 (optional): filter bandwidth (distance) between -3 dB points (Hz)
 /// - Output 0: filtered signal
-pub struct Resonator<T: Float, F: Real> {
+pub struct Resonator<T: Float, F: Real, N: Size<T>> {
+    _marker: std::marker::PhantomData<N>,
     biquad: Biquad<T, F>,
     sample_rate: F,
     center: F,
     bandwidth: F,
 }
 
-impl<T: Float, F: Real> Resonator<T, F> {
-    pub fn new(sample_rate: f64, center: F, bandwidth: F) -> Resonator<T, F> {
+impl<T: Float, F: Real, N: Size<T>> Resonator<T, F, N> {
+    pub fn new(sample_rate: f64, center: F, bandwidth: F) -> Self {
         let mut node = Resonator {
+            _marker: std::marker::PhantomData::default(),
             biquad: Biquad::new(),
             sample_rate: F::from_f64(sample_rate),
             center,
@@ -217,10 +225,10 @@ impl<T: Float, F: Real> Resonator<T, F> {
     }
 }
 
-impl<T: Float, F: Real> AudioNode for Resonator<T, F> {
+impl<T: Float, F: Real, N: Size<T>> AudioNode for Resonator<T, F, N> {
     const ID: u64 = 17;
     type Sample = T;
-    type Inputs = typenum::U3;
+    type Inputs = N;
     type Outputs = typenum::U1;
 
     fn reset(&mut self, sample_rate: Option<f64>) {
@@ -236,13 +244,15 @@ impl<T: Float, F: Real> AudioNode for Resonator<T, F> {
         &mut self,
         input: &Frame<Self::Sample, Self::Inputs>,
     ) -> Frame<Self::Sample, Self::Outputs> {
-        let center: F = convert(input[1]);
-        let bandwidth: F = convert(input[2]);
-        if center != self.center || bandwidth != self.bandwidth {
-            self.biquad
-                .set_coefs(BiquadCoefs::resonator(self.sample_rate, center, bandwidth));
-            self.center = center;
-            self.bandwidth = bandwidth;
+        if N::USIZE > 2 {
+            let center: F = convert(input[1]);
+            let bandwidth: F = convert(input[2]);
+            if center != self.center || bandwidth != self.bandwidth {
+                self.biquad
+                    .set_coefs(BiquadCoefs::resonator(self.sample_rate, center, bandwidth));
+                self.center = center;
+                self.bandwidth = bandwidth;
+            }
         }
         self.biquad.tick(&[input[0]].into())
     }
@@ -260,22 +270,23 @@ impl<T: Float, F: Real> AudioNode for Resonator<T, F> {
 }
 
 /// One-pole lowpass filter.
+/// The number of inputs is `N`, either `U1` or `U2`.
 /// - Input 0: input signal
-/// - Input 1: cutoff frequency (Hz)
+/// - Input 1 (optional): cutoff frequency (Hz)
 /// - Output 0: filtered signal
 #[derive(Default)]
-pub struct Lowpole<T: Float, F: Real> {
-    _marker: std::marker::PhantomData<T>,
+pub struct Lowpole<T: Float, F: Real, N: Size<T>> {
+    _marker: std::marker::PhantomData<(T, N)>,
     value: F,
     coeff: F,
     cutoff: F,
     sample_rate: F,
 }
 
-impl<T: Float, F: Real> Lowpole<T, F> {
+impl<T: Float, F: Real, N: Size<T>> Lowpole<T, F, N> {
     pub fn new(sample_rate: f64, cutoff: F) -> Self {
         let mut node = Lowpole {
-            _marker: std::marker::PhantomData,
+            _marker: std::marker::PhantomData::default(),
             value: F::zero(),
             coeff: F::zero(),
             cutoff,
@@ -290,10 +301,10 @@ impl<T: Float, F: Real> Lowpole<T, F> {
     }
 }
 
-impl<T: Float, F: Real> AudioNode for Lowpole<T, F> {
+impl<T: Float, F: Real, N: Size<T>> AudioNode for Lowpole<T, F, N> {
     const ID: u64 = 18;
     type Sample = T;
-    type Inputs = typenum::U2;
+    type Inputs = N;
     type Outputs = typenum::U1;
 
     fn reset(&mut self, sample_rate: Option<f64>) {
@@ -309,9 +320,11 @@ impl<T: Float, F: Real> AudioNode for Lowpole<T, F> {
         &mut self,
         input: &Frame<Self::Sample, Self::Inputs>,
     ) -> Frame<Self::Sample, Self::Outputs> {
-        let cutoff: F = convert(input[1]);
-        if cutoff != self.cutoff {
-            self.set_cutoff(cutoff);
+        if N::USIZE > 1 {
+            let cutoff: F = convert(input[1]);
+            if cutoff != self.cutoff {
+                self.set_cutoff(cutoff);
+            }
         }
         let x = convert(input[0]);
         self.value = (F::one() - self.coeff) * x + self.coeff * self.value;
@@ -400,7 +413,7 @@ fn logistic<T: Real>(x: T) -> T {
 }
 
 fn halfway_coeff<F: Real>(samples: F) -> F {
-    // This approximation is accurate to 0.5% when 1 <= response_samples <= 1.0e5.
+    // This approximation is accurate to 0.5% when 1 <= response_samples <= 500_000.
     let r0 = log(max(F::one(), samples)) - F::from_f64(0.861624594696583);
     let r1 = logistic(r0);
     let r2 = r1 * F::from_f64(1.13228543863477) - F::from_f64(0.1322853859);
@@ -410,7 +423,7 @@ fn halfway_coeff<F: Real>(samples: F) -> F {
 /// Smoothing filter with adjustable edge response time.
 /// - Input 0: input signal
 /// - Output 0: smoothed signal
-#[derive(Default, Clone)]
+#[derive(Default)]
 pub struct Follow<T: Float, F: Real> {
     v3: F,
     v2: F,
@@ -696,22 +709,24 @@ impl<T: Float, F: Float> AudioNode for Pinkpass<T, F> {
 }
 
 /// 1st order allpass filter.
+/// The number of inputs is `N`, either `U1` or `U2`.
 /// - Input 0: input signal
+/// - Input 1 (optional): delay in samples (delay > 0)
 /// - Output 0: filtered signal
 #[derive(Default)]
-pub struct Allpole<T: Float, F: Float> {
-    _marker: std::marker::PhantomData<T>,
+pub struct Allpole<T: Float, F: Float, N: Size<T>> {
+    _marker: std::marker::PhantomData<(T, N)>,
     eta: F,
     x1: F,
     y1: F,
     sample_rate: F,
 }
 
-impl<T: Float, F: Float> Allpole<T, F> {
+impl<T: Float, F: Float, N: Size<T>> Allpole<T, F, N> {
     pub fn new(sample_rate: f64, delay: F) -> Self {
         assert!(delay > F::zero());
         let mut node = Allpole {
-            _marker: std::marker::PhantomData,
+            _marker: std::marker::PhantomData::default(),
             eta: F::zero(),
             x1: F::zero(),
             y1: F::zero(),
@@ -721,15 +736,16 @@ impl<T: Float, F: Float> Allpole<T, F> {
         node
     }
 
+    #[inline]
     pub fn set_delay(&mut self, delay: F) {
         self.eta = F::one() - delay / (F::one() + delay);
     }
 }
 
-impl<T: Float, F: Float> AudioNode for Allpole<T, F> {
-    const ID: u64 = 45;
+impl<T: Float, F: Float, N: Size<T>> AudioNode for Allpole<T, F, N> {
+    const ID: u64 = 46;
     type Sample = T;
-    type Inputs = typenum::U1;
+    type Inputs = N;
     type Outputs = typenum::U1;
 
     fn reset(&mut self, sample_rate: Option<f64>) {
@@ -745,6 +761,9 @@ impl<T: Float, F: Float> AudioNode for Allpole<T, F> {
         &mut self,
         input: &Frame<Self::Sample, Self::Inputs>,
     ) -> Frame<Self::Sample, Self::Outputs> {
+        if N::USIZE > 1 {
+            self.set_delay(convert(input[1]));
+        }
         let x0 = convert(input[0]);
         let y0 = self.eta * (x0 - self.y1) + self.x1;
         self.x1 = x0;
@@ -764,12 +783,13 @@ impl<T: Float, F: Float> AudioNode for Allpole<T, F> {
 }
 
 /// One-pole, one-zero highpass filter.
+/// The number of inputs is `N`, either `U1` or `U2`.
 /// - Input 0: input signal
-/// - Input 1: cutoff frequency (Hz)
+/// - Input 1 (optional): cutoff frequency (Hz)
 /// - Output 0: filtered signal
 #[derive(Default)]
-pub struct Highpole<T: Float, F: Real> {
-    _marker: std::marker::PhantomData<T>,
+pub struct Highpole<T: Float, F: Real, N: Size<T>> {
+    _marker: std::marker::PhantomData<(T, N)>,
     x1: F,
     y1: F,
     coeff: F,
@@ -777,10 +797,10 @@ pub struct Highpole<T: Float, F: Real> {
     sample_rate: F,
 }
 
-impl<T: Float, F: Real> Highpole<T, F> {
+impl<T: Float, F: Real, N: Size<T>> Highpole<T, F, N> {
     pub fn new(sample_rate: f64, cutoff: F) -> Self {
         let mut node = Highpole {
-            _marker: std::marker::PhantomData,
+            _marker: std::marker::PhantomData::default(),
             x1: F::zero(),
             y1: F::zero(),
             coeff: F::zero(),
@@ -796,10 +816,10 @@ impl<T: Float, F: Real> Highpole<T, F> {
     }
 }
 
-impl<T: Float, F: Real> AudioNode for Highpole<T, F> {
-    const ID: u64 = 18;
+impl<T: Float, F: Real, N: Size<T>> AudioNode for Highpole<T, F, N> {
+    const ID: u64 = 47;
     type Sample = T;
-    type Inputs = typenum::U2;
+    type Inputs = N;
     type Outputs = typenum::U1;
 
     fn reset(&mut self, sample_rate: Option<f64>) {
@@ -816,9 +836,11 @@ impl<T: Float, F: Real> AudioNode for Highpole<T, F> {
         &mut self,
         input: &Frame<Self::Sample, Self::Inputs>,
     ) -> Frame<Self::Sample, Self::Outputs> {
-        let cutoff: F = convert(input[1]);
-        if cutoff != self.cutoff {
-            self.set_cutoff(cutoff);
+        if N::USIZE > 1 {
+            let cutoff: F = convert(input[1]);
+            if cutoff != self.cutoff {
+                self.set_cutoff(cutoff);
+            }
         }
         let x0 = convert(input[0]);
         let y0 = self.coeff * (self.y1 + x0 - self.x1);
