@@ -1,22 +1,23 @@
 //! The hacker prelude, a fully 64-bit environment for audio processing.
 
 pub use super::audionode::*;
+pub use super::audiounit::*;
 pub use super::combinator::*;
-pub use super::math::*;
-pub use super::*;
-
 pub use super::delay::*;
 pub use super::dynamics::*;
 pub use super::envelope::*;
 pub use super::feedback::*;
 pub use super::filter::*;
+pub use super::math::*;
 pub use super::noise::*;
 pub use super::oscillator::*;
+pub use super::sequencer::*;
 pub use super::shape::*;
 pub use super::signal::*;
 pub use super::svf::*;
 pub use super::wave::*;
 pub use super::wavetable::*;
+pub use super::*;
 
 pub use num_complex::Complex64;
 
@@ -163,14 +164,14 @@ pub fn multizero<U: Size<f64>>() -> An<Constant<f64, U>> {
 
 /// Mono pass-through.
 #[inline]
-pub fn pass() -> An<Pass<f64, U1>> {
+pub fn pass() -> An<Pass<f64>> {
     An(Pass::new())
 }
 
 /// Multichannel pass-through.
 #[inline]
-pub fn multipass<U: Size<f64>>() -> An<Pass<f64, U>> {
-    An(Pass::new())
+pub fn multipass<U: Size<f64>>() -> An<MultiPass<f64, U>> {
+    An(MultiPass::new())
 }
 
 /// Mono sink. Input is discarded.
@@ -214,36 +215,36 @@ pub fn sine_hz(f: f64) -> An<Pipe<f64, Constant<f64, U1>, Sine<f64>>> {
 #[inline]
 pub fn add<X: ConstantFrame<Sample = f64>>(
     x: X,
-) -> An<Binop<f64, FrameAdd<f64, X::Size>, Pass<f64, X::Size>, Constant<f64, X::Size>>>
+) -> An<Binop<f64, FrameAdd<f64, X::Size>, MultiPass<f64, X::Size>, Constant<f64, X::Size>>>
 where
     X::Size: Size<f64> + Add<U0>,
     <X::Size as Add<U0>>::Output: Size<f64>,
 {
-    An(Pass::<f64, X::Size>::new()) + dc(x)
+    An(MultiPass::<f64, X::Size>::new()) + dc(x)
 }
 
 /// Subtract constant from signal.
 #[inline]
 pub fn sub<X: ConstantFrame<Sample = f64>>(
     x: X,
-) -> An<Binop<f64, FrameSub<f64, X::Size>, Pass<f64, X::Size>, Constant<f64, X::Size>>>
+) -> An<Binop<f64, FrameSub<f64, X::Size>, MultiPass<f64, X::Size>, Constant<f64, X::Size>>>
 where
     X::Size: Size<f64> + Add<U0>,
     <X::Size as Add<U0>>::Output: Size<f64>,
 {
-    An(Pass::<f64, X::Size>::new()) - dc(x)
+    An(MultiPass::<f64, X::Size>::new()) - dc(x)
 }
 
 /// Multiply signal with constant.
 #[inline]
 pub fn mul<X: ConstantFrame<Sample = f64>>(
     x: X,
-) -> An<Binop<f64, FrameMul<f64, X::Size>, Pass<f64, X::Size>, Constant<f64, X::Size>>>
+) -> An<Binop<f64, FrameMul<f64, X::Size>, MultiPass<f64, X::Size>, Constant<f64, X::Size>>>
 where
     X::Size: Size<f64> + Add<U0>,
     <X::Size as Add<U0>>::Output: Size<f64>,
 {
-    An(Pass::<f64, X::Size>::new()) * dc(x)
+    An(MultiPass::<f64, X::Size>::new()) * dc(x)
 }
 
 /// Butterworth lowpass filter (2nd order).
@@ -251,7 +252,7 @@ where
 /// - Input 1: cutoff frequency (Hz)
 /// - Output 0: filtered audio
 #[inline]
-pub fn butterpass() -> An<ButterLowpass<f64, f64>> {
+pub fn butterpass() -> An<ButterLowpass<f64, f64, U2>> {
     An(ButterLowpass::new(DEFAULT_SR, 440.0))
 }
 
@@ -259,9 +260,7 @@ pub fn butterpass() -> An<ButterLowpass<f64, f64>> {
 /// - Input 0: audio
 /// - Output 0: filtered audio
 #[inline]
-pub fn butterpass_hz(
-    f: f64,
-) -> An<Pipe<f64, Stack<f64, Pass<f64, U1>, Constant<f64, U1>>, ButterLowpass<f64, f64>>> {
+pub fn butterpass_hz(f: f64) -> An<ButterLowpass<f64, f64, U1>> {
     super::prelude::butterpass_hz(f)
 }
 
@@ -270,7 +269,7 @@ pub fn butterpass_hz(
 /// - Input 1: cutoff frequency (Hz)
 /// - Output 0: filtered audio
 #[inline]
-pub fn lowpole() -> An<Lowpole<f64, f64>> {
+pub fn lowpole() -> An<Lowpole<f64, f64, U2>> {
     An(Lowpole::new(DEFAULT_SR, 440.0))
 }
 
@@ -278,17 +277,24 @@ pub fn lowpole() -> An<Lowpole<f64, f64>> {
 /// - Input 0: audio
 /// - Output 0: filtered audio
 #[inline]
-pub fn lowpole_hz(
-    f: f64,
-) -> An<Pipe<f64, Stack<f64, Pass<f64, U1>, Constant<f64, U1>>, Lowpole<f64, f64>>> {
+pub fn lowpole_hz(f: f64) -> An<Lowpole<f64, f64, U1>> {
     super::prelude::lowpole_hz(f)
 }
 
-/// Allpole filter with a configurable delay (delay > 0) in samples at DC.
+/// Allpole filter with adjustable delay (delay > 0) in samples at DC.
+/// - Input 0: audio
+/// - Input 1: delay in samples
+/// - Output 0: filtered audio
+#[inline]
+pub fn allpole() -> An<Allpole<f64, f64, U2>> {
+    An(Allpole::new(DEFAULT_SR, 1.0))
+}
+
+/// Allpole filter with delay (delay > 0) in samples at DC.
 /// - Input 0: audio
 /// - Output 0: filtered audio
 #[inline]
-pub fn allpole_delay(delay_in_samples: f64) -> An<Allpole<f64, f64>> {
+pub fn allpole_delay(delay_in_samples: f64) -> An<Allpole<f64, f64, U1>> {
     An(Allpole::new(DEFAULT_SR, delay_in_samples))
 }
 
@@ -297,7 +303,7 @@ pub fn allpole_delay(delay_in_samples: f64) -> An<Allpole<f64, f64>> {
 /// - Input 1: cutoff frequency (Hz)
 /// - Output 0: filtered audio
 #[inline]
-pub fn highpole() -> An<Highpole<f64, f64>> {
+pub fn highpole() -> An<Highpole<f64, f64, U2>> {
     An(Highpole::new(DEFAULT_SR, 440.0))
 }
 
@@ -305,10 +311,8 @@ pub fn highpole() -> An<Highpole<f64, f64>> {
 /// - Input 0: audio
 /// - Output 0: filtered audio
 #[inline]
-pub fn highpole_hz(
-    f: f64,
-) -> An<Pipe<f64, Stack<f64, Pass<f64, U1>, Constant<f64, U1>>, Highpole<f64, f64>>> {
-    (pass() | constant(f)) >> An(Highpole::new(DEFAULT_SR, f))
+pub fn highpole_hz(f: f64) -> An<Highpole<f64, f64, U1>> {
+    An(Highpole::new(DEFAULT_SR, f))
 }
 
 /// Constant-gain bandpass resonator.
@@ -317,7 +321,7 @@ pub fn highpole_hz(
 /// - Input 2: bandwidth (Hz)
 /// - Output 0: filtered audio
 #[inline]
-pub fn resonator() -> An<Resonator<f64, f64>> {
+pub fn resonator() -> An<Resonator<f64, f64, U3>> {
     An(Resonator::new(DEFAULT_SR, 440.0, 110.0))
 }
 
@@ -325,10 +329,7 @@ pub fn resonator() -> An<Resonator<f64, f64>> {
 /// - Input 0: audio
 /// - Output 0: filtered audio
 #[inline]
-pub fn resonator_hz(
-    center: f64,
-    bandwidth: f64,
-) -> An<Pipe<f64, Stack<f64, Pass<f64, U1>, Constant<f64, U2>>, Resonator<f64, f64>>> {
+pub fn resonator_hz(center: f64, bandwidth: f64) -> An<Resonator<f64, f64, U1>> {
     super::prelude::resonator_hz(center, bandwidth)
 }
 
@@ -528,16 +529,7 @@ pub fn pink() -> An<Pipe<f64, Noise<f64>, Pinkpass<f64, f64>>> {
 /// Brown noise.
 #[inline]
 pub fn brown() -> An<
-    Pipe<
-        f64,
-        Noise<f64>,
-        Binop<
-            f64,
-            FrameMul<f64, U1>,
-            Pipe<f64, Stack<f64, Pass<f64, U1>, Constant<f64, U1>>, Lowpole<f64, f64>>,
-            Constant<f64, U1>,
-        >,
-    >,
+    Pipe<f64, Noise<f64>, Binop<f64, FrameMul<f64, U1>, Lowpole<f64, f64, U1>, Constant<f64, U1>>>,
 > {
     // Empirical normalization factor.
     white() >> lowpole_hz(10.0) * dc(13.7)
@@ -553,7 +545,7 @@ pub fn goertzel() -> An<Goertzel<f64, f64>> {
 #[inline]
 pub fn goertzel_hz(
     f: f64,
-) -> An<Pipe<f64, Stack<f64, Pass<f64, U1>, Constant<f64, U1>>, Goertzel<f64, f64>>> {
+) -> An<Pipe<f64, Stack<f64, Pass<f64>, Constant<f64, U1>>, Goertzel<f64, f64>>> {
     super::prelude::goertzel_hz(f)
 }
 
@@ -793,18 +785,15 @@ pub fn reverb_stereo(
                                 f64,
                                 Pipe<
                                     f64,
-                                    Pipe<
-                                        f64,
-                                        Delay<f64>,
-                                        Pipe<
-                                            f64,
-                                            Stack<f64, Pass<f64, U1>, Constant<f64, U1>>,
-                                            Lowpole<f64, f64>,
-                                        >,
-                                    >,
+                                    Pipe<f64, Delay<f64>, Lowpole<f64, f64, U1>>,
                                     DCBlock<f64, f64>,
                                 >,
-                                Binop<f64, FrameMul<f64, U1>, Pass<f64, U1>, Constant<f64, U1>>,
+                                Binop<
+                                    f64,
+                                    FrameMul<f64, U1>,
+                                    MultiPass<f64, U1>,
+                                    Constant<f64, U1>,
+                                >,
                             >,
                         >,
                         U32,
@@ -813,9 +802,9 @@ pub fn reverb_stereo(
                 >,
                 MultiJoin<f64, U2, U16>,
             >,
-            Binop<f64, FrameMul<f64, U2>, Pass<f64, U2>, Constant<f64, U2>>,
+            Binop<f64, FrameMul<f64, U2>, MultiPass<f64, U2>, Constant<f64, U2>>,
         >,
-        Binop<f64, FrameMul<f64, U2>, Pass<f64, U2>, Constant<f64, U2>>,
+        Binop<f64, FrameMul<f64, U2>, MultiPass<f64, U2>, Constant<f64, U2>>,
     >,
 > {
     super::prelude::reverb_stereo::<f64, f64>(wet, time)
@@ -885,7 +874,8 @@ pub fn lowpass_hz(f: f64, q: f64) -> An<FixedSvf<f64, f64, LowpassMode<f64>>> {
 #[inline]
 pub fn lowpass_q(
     q: f64,
-) -> An<Pipe<f64, Stack<f64, Pass<f64, U2>, Constant<f64, U1>>, Svf<f64, f64, LowpassMode<f64>>>> {
+) -> An<Pipe<f64, Stack<f64, MultiPass<f64, U2>, Constant<f64, U1>>, Svf<f64, f64, LowpassMode<f64>>>>
+{
     super::prelude::lowpass_q::<f64, f64>(q)
 }
 
@@ -914,7 +904,9 @@ pub fn highpass_hz(f: f64, q: f64) -> An<FixedSvf<f64, f64, HighpassMode<f64>>>
 #[inline]
 pub fn highpass_q(
     q: f64,
-) -> An<Pipe<f64, Stack<f64, Pass<f64, U2>, Constant<f64, U1>>, Svf<f64, f64, HighpassMode<f64>>>> {
+) -> An<
+    Pipe<f64, Stack<f64, MultiPass<f64, U2>, Constant<f64, U1>>, Svf<f64, f64, HighpassMode<f64>>>,
+> {
     super::prelude::highpass_q::<f64, f64>(q)
 }
 
@@ -943,7 +935,9 @@ pub fn bandpass_hz(f: f64, q: f64) -> An<FixedSvf<f64, f64, BandpassMode<f64>>>
 #[inline]
 pub fn bandpass_q(
     q: f64,
-) -> An<Pipe<f64, Stack<f64, Pass<f64, U2>, Constant<f64, U1>>, Svf<f64, f64, BandpassMode<f64>>>> {
+) -> An<
+    Pipe<f64, Stack<f64, MultiPass<f64, U2>, Constant<f64, U1>>, Svf<f64, f64, BandpassMode<f64>>>,
+> {
     super::prelude::bandpass_q::<f64, f64>(q)
 }
 
@@ -972,7 +966,8 @@ pub fn notch_hz(f: f64, q: f64) -> An<FixedSvf<f64, f64, NotchMode<f64>>> {
 #[inline]
 pub fn notch_q(
     q: f64,
-) -> An<Pipe<f64, Stack<f64, Pass<f64, U2>, Constant<f64, U1>>, Svf<f64, f64, NotchMode<f64>>>> {
+) -> An<Pipe<f64, Stack<f64, MultiPass<f64, U2>, Constant<f64, U1>>, Svf<f64, f64, NotchMode<f64>>>>
+{
     super::prelude::notch_q::<f64, f64>(q)
 }
 
@@ -1001,7 +996,8 @@ pub fn peak_hz(f: f64, q: f64) -> An<FixedSvf<f64, f64, PeakMode<f64>>> {
 #[inline]
 pub fn peak_q(
     q: f64,
-) -> An<Pipe<f64, Stack<f64, Pass<f64, U2>, Constant<f64, U1>>, Svf<f64, f64, PeakMode<f64>>>> {
+) -> An<Pipe<f64, Stack<f64, MultiPass<f64, U2>, Constant<f64, U1>>, Svf<f64, f64, PeakMode<f64>>>>
+{
     super::prelude::peak_q::<f64, f64>(q)
 }
 
@@ -1030,7 +1026,8 @@ pub fn allpass_hz(f: f64, q: f64) -> An<FixedSvf<f64, f64, AllpassMode<f64>>> {
 #[inline]
 pub fn allpass_q(
     q: f64,
-) -> An<Pipe<f64, Stack<f64, Pass<f64, U2>, Constant<f64, U1>>, Svf<f64, f64, AllpassMode<f64>>>> {
+) -> An<Pipe<f64, Stack<f64, MultiPass<f64, U2>, Constant<f64, U1>>, Svf<f64, f64, AllpassMode<f64>>>>
+{
     super::prelude::allpass_q::<f64, f64>(q)
 }
 
@@ -1061,7 +1058,8 @@ pub fn bell_hz(f: f64, q: f64, gain: f64) -> An<FixedSvf<f64, f64, BellMode<f64>
 pub fn bell_q(
     q: f64,
     gain: f64,
-) -> An<Pipe<f64, Stack<f64, Pass<f64, U2>, Constant<f64, U2>>, Svf<f64, f64, BellMode<f64>>>> {
+) -> An<Pipe<f64, Stack<f64, MultiPass<f64, U2>, Constant<f64, U2>>, Svf<f64, f64, BellMode<f64>>>>
+{
     super::prelude::bell_q::<f64, f64>(q, gain)
 }
 
@@ -1092,7 +1090,9 @@ pub fn lowshelf_hz(f: f64, q: f64, gain: f64) -> An<FixedSvf<f64, f64, LowshelfM
 pub fn lowshelf_q(
     q: f64,
     gain: f64,
-) -> An<Pipe<f64, Stack<f64, Pass<f64, U2>, Constant<f64, U2>>, Svf<f64, f64, LowshelfMode<f64>>>> {
+) -> An<
+    Pipe<f64, Stack<f64, MultiPass<f64, U2>, Constant<f64, U2>>, Svf<f64, f64, LowshelfMode<f64>>>,
+> {
     super::prelude::lowshelf_q::<f64, f64>(q, gain)
 }
 
@@ -1123,8 +1123,9 @@ pub fn highshelf_hz(f: f64, q: f64, gain: f64) -> An<FixedSvf<f64, f64, Highshel
 pub fn highshelf_q(
     q: f64,
     gain: f64,
-) -> An<Pipe<f64, Stack<f64, Pass<f64, U2>, Constant<f64, U2>>, Svf<f64, f64, HighshelfMode<f64>>>>
-{
+) -> An<
+    Pipe<f64, Stack<f64, MultiPass<f64, U2>, Constant<f64, U2>>, Svf<f64, f64, HighshelfMode<f64>>>,
+> {
     super::prelude::highshelf_q::<f64, f64>(q, gain)
 }
 
